//
//  ConfigurarTaxaEditarTableViewController.swift
//  LloguersTuristics
//
//  Created by Manel Juarez on 2/5/15.
//  Copyright (c) 2015 Manel Juarez. All rights reserved.
//

import UIKit
import CoreData

class ConfigurarTaxaEditarTableViewController: UITableViewController, UITextFieldDelegate {

    // Obtenir el Context per el Core Data
    let moContext = (UIApplication.sharedApplication().delegate as AppDelegate).managedObjectContext!
    
    // Objecte que conté les dades de la taxa actual que estem editant
    // Si el valor és nil significa que estem creant un nou registre
    // És el Model d'aquest MVC
    var moTaxaActual: Configuracio?
    
    // Botons per guardar i cancelar
    @IBOutlet weak var btnGuardar: UIBarButtonItem!
    @IBOutlet weak var btnCancelar: UIBarButtonItem!
    
    // Textfields per editar valors que tenim definits a la pantalla d'edició
    @IBOutlet weak var tfTaxaPersonaNit: UITextField!
    @IBOutlet weak var tfDataValidesa: UITextField!
    @IBOutlet weak var tfZona: UITextField!
    
    // Steppers que tenim definits per els camps numèrics
    @IBOutlet weak var stepperZona: UIStepper!

    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Evitar que el tableview mostri les files en blanc al final, ja que és un model estàtic
        // per simular un formulari d'edició
        tableView.tableFooterView = UIView(frame: CGRect.zeroRect)
        
        // Assignar el delegate dels textfield, per poder gestionar el return = next etc...
        tfDataValidesa.delegate = self
        tfZona.delegate = self
        tfTaxaPersonaNit.delegate = self
        
        // Carregar els camps amb els valors de la taxa si estem modificant.
        // La data de validesa s'ha de formatar per poder-la mostrar
        var dateformatter: NSDateFormatter = NSDateFormatter()
        dateformatter.dateStyle = NSDateFormatterStyle.LongStyle
        dateformatter.locale = NSLocale.currentLocale()
        
        // Cal tenir en compte la localització (per exemple en català/español el decimal és una coma, no un punt. Afagem el currentLocale de l'aparell de l'usuari
        var formatter = NSNumberFormatter()
        formatter.locale = NSLocale.currentLocale()
        formatter.numberStyle = NSNumberFormatterStyle.DecimalStyle
        // Màxim dos decimals als imports
        formatter.maximumFractionDigits = 2
        
        if ((moTaxaActual) != nil) {
            var datavalidesa = moTaxaActual?.datavalidesa
            tfDataValidesa?.text = dateformatter.stringFromDate(datavalidesa!) as String?
            tfZona?.text = moTaxaActual?.zona==nil ? "0" : moTaxaActual?.zona.stringValue
            tfTaxaPersonaNit?.text = formatter.stringFromNumber(moTaxaActual?.taxapersonanit as Double)
            
            // Carregar el valor dels steppers
            stepperZona.value = moTaxaActual?.zona==nil ? 0 : moTaxaActual?.zona as Double
            
        } else {
            
            // Si estem en mode crear
            // Posem data rellotge i altres valors per defecte
            tfDataValidesa?.text = dateformatter.stringFromDate(NSDate()) as String?
            tfZona?.text = "1"
            stepperZona.value = 1
            
            // passem focus al segon camp perquè desplegui el teclat
            tfTaxaPersonaNit.becomeFirstResponder()
        }
   }


    // MARK: - Table view data source
    
    // Retornar el total de seccions
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }
    
    // Retornar el total de files per la secció indicada
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return 1
    }
    
    // El camp de la data d'entrada l'editarem amb un Date Picker
    @IBAction func tfDataEditDidBegin(sender: UITextField) {
        var datePickerView:UIDatePicker = UIDatePicker()
        datePickerView.datePickerMode = UIDatePickerMode.Date
        
        // Assignar la data actual del camp en pantalla al picker
        var dateformatter: NSDateFormatter = NSDateFormatter()
        dateformatter.dateStyle = NSDateFormatterStyle.LongStyle
        dateformatter.locale = NSLocale.currentLocale()
        datePickerView.date = dateformatter.dateFromString(tfDataValidesa.text)!
        
        sender.inputView = datePickerView
        datePickerView.addTarget(self, action: Selector("datePickerValueChanged:"), forControlEvents: UIControlEvents.ValueChanged)
    }
    
    // Un cop seleccionada una data amb el Date Picker l'assignem al UITextfield associat
    func datePickerValueChanged(sender: UIDatePicker) {
        var dateformatter = NSDateFormatter()
        dateformatter.dateStyle = NSDateFormatterStyle.LongStyle
        dateformatter.locale = NSLocale.currentLocale()
        tfDataValidesa.text = dateformatter.stringFromDate(sender.date)
    }
    
    // Gestionar els steppers per modificar els camps numèrics
    @IBAction func stepperZonaChange(sender: UIStepper) {
        self.tfZona.text = Int(sender.value).description
        amagarTeclat()
    }

    //
    // Amagar teclat
    //
    func amagarTeclat() {
        tfDataValidesa.resignFirstResponder()
        tfTaxaPersonaNit.resignFirstResponder()
    }
    
    //
    // Gestionar el botó return del teclat per canviar de camp
    //
    func textFieldShouldReturn(textField: UITextField) -> Bool {
        switch tfDataValidesa {
        case self.tfDataValidesa:
            self.tfTaxaPersonaNit.becomeFirstResponder()
            break
        case self.tfTaxaPersonaNit:
            self.tfDataValidesa.becomeFirstResponder()
            break
        default:
            break
        }
        return true
    }
    
    // Els camps que tenen un stepper associat no els editem manualment
    func textFieldShouldBeginEditing(textField: UITextField) -> Bool {
        switch textField {
        case self.tfZona:
            return false
        default:
            return true
        }
    }
    

    // MARK: - Navigation
    
    // Validacions abans de guardar
    override func shouldPerformSegueWithIdentifier(identifier: String!, sender: AnyObject!) -> Bool {
        
        if (sender as UIBarButtonItem == self.btnGuardar) {
            
            // La data no cal validar-la, s'introdueix amb picker i té valor per defecte.
            // La zona no cal validar-la el input es gestiona amb "steppers"
            
            // Validar format del import (separador decimal, només números , etc...)
            // Cal tenir en compte la localització per el decimal
            var formatter = NSNumberFormatter()
            formatter.locale = NSLocale.currentLocale()
            formatter.numberStyle = NSNumberFormatterStyle.DecimalStyle
            // Màxim 2 decimals
            formatter.maximumFractionDigits = 2
            // Convertir de cadena a tipus NSDecimalNumber per guardar al Core Data
            if ((formatter.numberFromString(tfTaxaPersonaNit.text)) == nil) {
                // Import incorrecte
                let alert = UIAlertView()
                alert.title = NSLocalizedString("Validation", comment: "Validació import")
                alert.message = NSLocalizedString("Incorrect amount", comment: "Validació import incorrecte")
                alert.addButtonWithTitle("Ok")
                alert.show()
                // Validació incorrecte
                return false
            }
        }
        
        // Validació correcte
        return true
    }
    
    
    //
    // Atendre les peticions dels Segue que tenim definits a l'storyboard
    //
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        
        // Preparar per el segue (unwind) que torna enrera (cancelar o guardar)
        if (sender as UIBarButtonItem == self.btnGuardar) {
            if ((moTaxaActual) == nil) {
                // Crear nou model si estem "insertant"
                // Obtenir el entity Configuracio
                let entityConfig = NSEntityDescription.entityForName("Configuracio", inManagedObjectContext: self.moContext)!
                //  Crear un nou objecte Configuracio
                self.moTaxaActual = Configuracio(entity: entityConfig, insertIntoManagedObjectContext: self.moContext)
            }
            // Passar les dades dels camps d'edició al model moTaxaActual
            // La data s'ha de formatar per poder-la guardar
            var dateformatter: NSDateFormatter = NSDateFormatter()
            dateformatter.dateStyle = NSDateFormatterStyle.LongStyle
            dateformatter.locale = NSLocale.currentLocale()
            self.moTaxaActual?.datavalidesa = dateformatter.dateFromString(tfDataValidesa.text)!
            
            self.moTaxaActual?.zona = tfZona.text.toInt()!
            
            // L'import es valida prèviament per tant el valor entrar és vàlid en aquest punt
            if (tfTaxaPersonaNit.text.isEmpty || tfTaxaPersonaNit.text==nil) {
                self.moTaxaActual?.taxapersonanit =  0.00
            } else {
                // Cal tenir en compte la localització per el decimal
                var formatter = NSNumberFormatter()
                formatter.locale = NSLocale.currentLocale()
                formatter.numberStyle = NSNumberFormatterStyle.DecimalStyle
                // Màxim 2 decimals
                formatter.maximumFractionDigits = 2
                // Convertir de cadena a tipus NSDecimalNumber per guardar al Core Data
                self.moTaxaActual?.taxapersonanit = NSDecimalNumber(double: formatter.numberFromString(tfTaxaPersonaNit.text)!.doubleValue)
            }
            
            
            // Guardar context
            var errorGuardar: NSError? = nil
            if !moContext.save(&errorGuardar) {
                println(NSLocalizedString("Error saving tax settings:", comment: "Errors control") + " \(errorGuardar!.localizedDescription)")
                abort();
            }
            
        }
    }

}
