//
//  DespesaEditarTableViewController.swift
//  LloguersTuristics
//
//  Created by Manel Juarez on 6/4/15.
//  Copyright (c) 2015 Manel Juarez. All rights reserved.
//

import UIKit
import CoreData

class DespesaEditarTableViewController: UITableViewController, UITextFieldDelegate {

    // Obtenir el Context per el Core Data
    let moContext = (UIApplication.sharedApplication().delegate as AppDelegate).managedObjectContext!
    
    // Objecte que conté les dades del habitatge seleccionat.
    var moHabitatgeActual: Habitatges?
    // Objecte que conté les dades de la despesa actual que estem editant
    // Si el valor és nil significa que estem creant una nova despesa
    // És el Model d'aquest MVC
    var moDespesaActual: Despeses?
    
    // Botons per guardar i cancelar
    @IBOutlet weak var btnCancelar: UIBarButtonItem!
    @IBOutlet weak var btnGuardar: UIBarButtonItem!
    
    // Textfields per editar valors que tenim definits a la pantalla d'edició
    @IBOutlet weak var tfDatafactura: UITextField!
    @IBOutlet weak var tfConcepte: UITextField!
    @IBOutlet weak var tfImportpagat: UITextField!
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Evitar que el tableview mostri les files en blanc al final, ja que és un model estàtic
        // per simular un formulari d'edició
        tableView.tableFooterView = UIView(frame: CGRect.zeroRect)
        
        // Assignar el delegate dels textfield, per poder gestionar el return = next etc...
        tfDatafactura.delegate = self
        tfConcepte.delegate = self
        tfImportpagat.delegate = self
        
        // Carregar els camps amb els valors de la despesa si estem modificant.
        // La data de la despesa s'ha de formatar per poder-la mostrar
        var dateformatter: NSDateFormatter = NSDateFormatter()
        dateformatter.dateStyle = NSDateFormatterStyle.LongStyle
        dateformatter.locale = NSLocale.currentLocale()
        
        // Cal tenir en compte la localització (per exemple en català/español el decimal és una coma, no un punt. Afagem el currentLocale de l'aparell de l'usuari
        var formatter = NSNumberFormatter()
        formatter.locale = NSLocale.currentLocale()
        formatter.numberStyle = NSNumberFormatterStyle.DecimalStyle
        // Màxim dos decimals als imports
        formatter.maximumFractionDigits = 2
        
        if ((moDespesaActual) != nil) {
            var datafactura = moDespesaActual?.datafactura
            tfDatafactura?.text = dateformatter.stringFromDate(datafactura!) as String?
            tfConcepte?.text = moDespesaActual?.concepte
            tfImportpagat?.text = formatter.stringFromNumber(moDespesaActual?.importpagat as Double)
            
        } else {
            
            // Si estem en mode crear
            // Posem data rellotge i altres valors per defecte
            tfDatafactura?.text = dateformatter.stringFromDate(NSDate()) as String?
            tfConcepte?.text = ""
            
            // passem focus al segon camp perquè desplegui el teclat
            tfConcepte.becomeFirstResponder()
        }
    }
    
    // MARK: - Table view data source
    
    // Retornar el total de seccions
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }
    
    // Retornar el total de files per la secció indicada
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return 1
    }
    
    // Assignar títol i estil (color) al fons de la única secció
    override func tableView(tableView: UITableView, willDisplayHeaderView view: UIView, forSection section: Int) {
        
        // Fem un cast per el tipus de la capçalera
        let header: UITableViewHeaderFooterView = view as UITableViewHeaderFooterView
        
        if (section==0) {
            // Posar colo de fons
            header.contentView.backgroundColor = UIColor(red: 108.0/255.0, green: 226.0/255.0, blue: 108.0/255.0, alpha: 0.5)
            // Posar nom de l'habitatge com a títol
            header.textLabel.text = moHabitatgeActual?.nom
        }
    }
    
    
    // El camp de la data d'entrada l'editarem amb un Date Picker
    @IBAction func tfDatafacturaEditDidBegin(sender: UITextField) {
        var datePickerView:UIDatePicker = UIDatePicker()
        datePickerView.datePickerMode = UIDatePickerMode.Date
        
        // Assignar la data actual del camp en pantalla al picker
        var dateformatter: NSDateFormatter = NSDateFormatter()
        dateformatter.dateStyle = NSDateFormatterStyle.LongStyle
        dateformatter.locale = NSLocale.currentLocale()
        datePickerView.date = dateformatter.dateFromString(tfDatafactura.text)!
        
        sender.inputView = datePickerView
        datePickerView.addTarget(self, action: Selector("datePickerValueChanged:"), forControlEvents: UIControlEvents.ValueChanged)
    }
    
    // Un cop seleccionada una data amb el Date Picker l'assignem al UITextfield associat
    func datePickerValueChanged(sender: UIDatePicker) {
        var dateformatter = NSDateFormatter()
        dateformatter.dateStyle = NSDateFormatterStyle.LongStyle
        dateformatter.locale = NSLocale.currentLocale()
        tfDatafactura.text = dateformatter.stringFromDate(sender.date)
    }
    
    //
    // Gestionar el botó return del teclat per canviar de camp
    //
    func textFieldShouldReturn(textField: UITextField) -> Bool {
        switch textField {
        case self.tfDatafactura:
            self.tfConcepte.becomeFirstResponder()
            break
        case self.tfConcepte:
            self.tfImportpagat.becomeFirstResponder()
            break
        default:
            break
        }
        return true
    }
    
    
    // MARK: - Navigation

    // Validacions abans de guardar
    override func shouldPerformSegueWithIdentifier(identifier: String!, sender: AnyObject!) -> Bool {

        if (sender as UIBarButtonItem == self.btnGuardar) {
        
            // La data no cal validar-la, s'introdueix amb picker i té valor per defecte.
            
            // Validar camp text concepte (no pot estar buit)
            if (tfConcepte.text.isEmpty || tfConcepte.text==nil) {
                // Concepte incorrecte
                let alert = UIAlertView()
                alert.title = NSLocalizedString("Validation", comment: "Validació concepte despesa")
                alert.message = NSLocalizedString("Concept cannot be empty", comment: "Validació concepte  incorrecte")
                alert.addButtonWithTitle("Ok")
                alert.show()
                // Validació incorrecte
                return false
            }
            
            // Validar format del import (separador decimal, només números , etc...)
            // Cal tenir en compte la localització per el decimal
            var formatter = NSNumberFormatter()
            formatter.locale = NSLocale.currentLocale()
            formatter.numberStyle = NSNumberFormatterStyle.DecimalStyle
            // Màxim 2 decimals
            formatter.maximumFractionDigits = 2
            // Convertir de cadena a tipus NSDecimalNumber per guardar al Core Data
            if ((formatter.numberFromString(tfImportpagat.text)) == nil) {
                // Import incorrecte
                let alert = UIAlertView()
                alert.title = NSLocalizedString("Validation", comment: "Validació import")
                alert.message = NSLocalizedString("Incorrect amount", comment: "Validació import incorrecte")
                alert.addButtonWithTitle("Ok")
                alert.show()
                // Validació incorrecte
                return false
            }
        }
        
        // Validació correcte
        return true
    }
    
    
    //
    // Atendre les peticions dels Segue que tenim definits a l'storyboard
    //
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        
        // Preparar per el segue (unwind) que torna enrera (cancelar o guardar)
        if (sender as UIBarButtonItem == self.btnGuardar) {
            if ((moDespesaActual) == nil) {
                // Crear nou model si estem "insertant"
                // Obtenir el entity Ingressos
                let entityDespeses = NSEntityDescription.entityForName("Despeses", inManagedObjectContext: self.moContext)!
                //  Crear un nou objecte Despeses
                self.moDespesaActual = Despeses(entity: entityDespeses, insertIntoManagedObjectContext: self.moContext)
                // Vincular amb l'habitatge actual
                self.moDespesaActual?.habitatge = self.moHabitatgeActual!
            }
            // Passar les dades dels camps d'edició al model moDespesaActual
            // La data de l'ingrés s'ha de formatar per poder-la guardar
            var dateformatter: NSDateFormatter = NSDateFormatter()
            dateformatter.dateStyle = NSDateFormatterStyle.LongStyle
            dateformatter.locale = NSLocale.currentLocale()
            self.moDespesaActual?.datafactura = dateformatter.dateFromString(tfDatafactura.text)!
            
            self.moDespesaActual?.concepte = tfConcepte.text
            
            // L'import es valida prèviament per tant el valor entrar és vàlid en aquest punt
            if (tfImportpagat.text.isEmpty || tfImportpagat.text==nil) {
                self.moDespesaActual?.importpagat =  0.00
            } else {
                // Cal tenir en compte la localització per el decimal
                var formatter = NSNumberFormatter()
                formatter.locale = NSLocale.currentLocale()
                formatter.numberStyle = NSNumberFormatterStyle.DecimalStyle
                // Màxim 2 decimals
                formatter.maximumFractionDigits = 2
                // Convertir de cadena a tipus NSDecimalNumber per guardar al Core Data
                self.moDespesaActual?.importpagat = NSDecimalNumber(double: formatter.numberFromString(tfImportpagat.text)!.doubleValue)
            }
            
            
            // Guardar context
            var errorGuardar: NSError? = nil
            if !moContext.save(&errorGuardar) {
                println(NSLocalizedString("Error saving expense:", comment: "Errors control") + " \(errorGuardar!.localizedDescription)")
                abort();
            }
            
            // Refrescar l'objecte de l'habitatge actual contra la base de dades
            moContext.refreshObject(moHabitatgeActual!, mergeChanges: true)
        }
    }


}
