//
//  HabitatgeEditarTableViewController.swift
//  LloguersTuristics
//
//  Created by Manel Juarez on 28/3/15.
//  Copyright (c) 2015 Manel Juarez. All rights reserved.
//

import UIKit
import CoreData

class HabitatgeEditarTableViewController: UITableViewController, UITextFieldDelegate {

    // Obtenir el Context per el Core Data
    let moContext = (UIApplication.sharedApplication().delegate as AppDelegate).managedObjectContext!
    
    // Objecte que conté les dades del habitatge actual que estem editant.
    // Si el valor és nil significa que estem creant un nou habitatge
    // És el Model d'aquest MVC
    var moHabitatgeActual: Habitatges?
    
    // Botons guardar i cancelar
    @IBOutlet weak var btnGuardar: UIBarButtonItem!
    @IBOutlet weak var btnCancelar: UIBarButtonItem!
    
    // Textfields per editar valors que tenim definits a la pantalla d'edició
    @IBOutlet weak var tfNom: UITextField!
    @IBOutlet weak var tfDireccio: UITextField!
    @IBOutlet weak var tfPoblacio: UITextField!
    @IBOutlet weak var tfZona: UITextField!
    
    // Steppers que tenim definits per els camps numèrics
    @IBOutlet weak var stepperZona: UIStepper!
    
    // Al carregar 
    override func viewDidLoad() {
        super.viewDidLoad()

        // Evitar que el tableview mostri les files en blanc al final, ja que és un disseny estàtic
        // per simular un formulari d'edició
        tableView.tableFooterView = UIView(frame: CGRect.zeroRect)

        // Assignar el delegate dels textfield, per poder gestionar el return = next etc...
        tfNom.delegate = self
        tfDireccio.delegate = self
        tfPoblacio.delegate = self
        tfZona.delegate = self

        // Carregar els camps amb els valors de l'habitatge si estem modificant.
        if ((moHabitatgeActual) != nil) {
            tfNom?.text = moHabitatgeActual?.nom
            tfDireccio?.text = moHabitatgeActual?.direccio
            tfPoblacio?.text = moHabitatgeActual?.poblacio
            tfZona?.text = moHabitatgeActual?.zona==nil ? "0" : moHabitatgeActual?.zona.stringValue
            // Carregar el valor dels steppers
            stepperZona.value = moHabitatgeActual?.zona==nil ? 0 : moHabitatgeActual?.zona as Double
        } else {
            // Si estem en mode crear, passem focus al primer camp perquè desplegui el teclat
            tfNom.becomeFirstResponder()
        }
    }

    // Gestionar els steppers per modificar els camps numèrics
    @IBAction func stepperZonaChange(sender: UIStepper) {
        self.tfZona.text = Int(sender.value).description
        amagarTeclat()
    }
   
    //
    // Amagar teclat
    //
    func amagarTeclat() {
        tfNom.resignFirstResponder()
        tfDireccio.resignFirstResponder()
        tfPoblacio.resignFirstResponder()
    }

    // Els camps que tenen un stepper associat no els editem manualment
    func textFieldShouldBeginEditing(textField: UITextField) -> Bool {
        switch textField {
        case self.tfZona:
            return false
        default:
            return true
        }
    }

    
    // MARK: - Table view data source

    // Retornar el total de seccions
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }

    // Retornar el total de files per la secció indicada
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return 1
    }

    // Assignar estil (color) al fons de les seccions
    override func tableView(tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        var header :UITableViewHeaderFooterView = UITableViewHeaderFooterView()
        header.contentView.backgroundColor = UIColor(red: 108.0/255.0, green: 226.0/255.0, blue: 108.0/255.0, alpha: 0.5)
        return header
    }
    
    // MARK: - Navigation

    // Validacions abans de guardar
    override func shouldPerformSegueWithIdentifier(identifier: String!, sender: AnyObject!) -> Bool {
        
        if (sender as UIBarButtonItem == self.btnGuardar) {
            
            // Validar camp text nom (no pot estar buit)
            if (tfNom.text.isEmpty || tfNom.text==nil) {
                // Nom incorrecte
                let alert = UIAlertView()
                alert.title = NSLocalizedString("Validation", comment: "Validació nom habitatge")
                alert.message = NSLocalizedString("Name cannot be empty", comment: "Validació nom habitatge")
                alert.addButtonWithTitle("Ok")
                alert.show()
                // Validació incorrecte
                return false
            }

            // Validar camp text adreça (no pot estar buit)
            if (tfDireccio.text.isEmpty || tfDireccio.text==nil) {
                // Adreça incorrecte
                let alert = UIAlertView()
                alert.title = NSLocalizedString("Validation", comment: "Validació adreça habitatge")
                alert.message = NSLocalizedString("Address cannot be empty", comment: "Validació adreça habitatge")
                alert.addButtonWithTitle("Ok")
                alert.show()
                // Validació incorrecte
                return false
            }

            // Validar camp text població (no pot estar buit)
            if (tfPoblacio.text.isEmpty || tfPoblacio.text==nil) {
                // Població incorrecte
                let alert = UIAlertView()
                alert.title = NSLocalizedString("Validation", comment: "Validació població habitatge")
                alert.message = NSLocalizedString("Town cannot be empty", comment: "Validació població habitatge")
                alert.addButtonWithTitle("Ok")
                alert.show()
                // Validació incorrecte
                return false
            }

        }
        
        // Validació correcte
        return true
    }
    
    

    //
    // Atendre les peticions dels Segue que tenim definits a l'storyboard
    //
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
       
        // Preparar per el segue (unwind) que torna enrera (cancelar o guardar)
        if (sender as UIBarButtonItem == self.btnGuardar) {
            if ((moHabitatgeActual) == nil) {
                // Crear nou model si estem "insertant"
                // Obtenir el entity Habitatges
                let entityHabitatges = NSEntityDescription.entityForName("Habitatges", inManagedObjectContext: self.moContext)!
                //  Crear un nou objecte Habitatge
                self.moHabitatgeActual = Habitatges(entity: entityHabitatges, insertIntoManagedObjectContext: self.moContext)
            }
            
            // Passar les dades dels camps d'edició al model moHabitatgeActual
            self.moHabitatgeActual?.nom = tfNom.text
            self.moHabitatgeActual?.direccio = tfDireccio.text
            self.moHabitatgeActual?.poblacio = tfPoblacio.text
            self.moHabitatgeActual?.zona = tfZona.text.toInt()!
            
            // Guardar context
            var errorGuardar: NSError? = nil
            if !moContext.save(&errorGuardar) {
                println(NSLocalizedString("Error saving accommodation data:", comment: "Errors control") + "  \(errorGuardar!.localizedDescription)")
                abort();
            }
        }
    }

    // 
    // Gestionar el botó return del teclat per canviar de camp
    //
    func textFieldShouldReturn(textField: UITextField) -> Bool {
        switch textField {
        case self.tfNom:
            self.tfDireccio.becomeFirstResponder()
            break
        case self.tfDireccio:
            self.tfPoblacio.becomeFirstResponder()
            break
        case self.tfPoblacio:
            self.tfNom.becomeFirstResponder()
            break
        default:
            break
        }
        return true
    }
    
}
