//
//  HabitatgesDetailTableViewController.swift
//  LloguersTuristics
//
//  Created by Manel Juarez on 8/3/15.
//  Copyright (c) 2015 Manel Juarez. All rights reserved.
//

import UIKit
import CoreData

class HabitatgesDetailTableViewController: UITableViewController {

    // Obtenir el Context per el Core Data
    let moContext = (UIApplication.sharedApplication().delegate as AppDelegate).managedObjectContext!
    
    // Objecte que conté les dades de l'habitatge actual que estem mostrant.
    // És el Model d'aquest MVC
    var moHabitatgeActual: Habitatges?

    // Definir variables pels totals
    var totalIngressos: NSDecimalNumber = NSDecimalNumber.zero()
    var totalDespeses: NSDecimalNumber = NSDecimalNumber.zero()
    var totalTaxes: NSDecimalNumber = NSDecimalNumber.zero()
    var totalBenefici: NSDecimalNumber = NSDecimalNumber.zero()
    
    // Cel.les definides de forma estàtica
    @IBOutlet weak var lblNom: UILabel!
    @IBOutlet weak var lblDireccio: UILabel!
    @IBOutlet weak var lblPoblacio: UILabel!
    
    @IBOutlet weak var lblTotalIngressos: UILabel!
    @IBOutlet weak var lblTotalDespeses: UILabel!
    @IBOutlet weak var lblTotalTaxes: UILabel!
    @IBOutlet weak var lblTotalBenefici: UILabel!
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Afegir botons de navegació per poder tornar enrera
        navigationItem.leftBarButtonItem = splitViewController?.displayModeButtonItem()
        navigationItem.leftItemsSupplementBackButton = true
        
        // Mostrar dades habitatge
        mostrarDadesHabitatge()
    }

    // Cada cop que es mostra la vista cal refrescar els totals
    override func viewDidAppear(animated: Bool) {
        // Calcular els totals de ingressos, despeses i taxes
        calcularTotalsHabitatge()
    }
    
    @IBAction func refrescarLlista(sender: UIRefreshControl) {
        // Refrescar els totals
        calcularTotalsHabitatge()        
        // Amagar la imatge "spinning-carregant"
        sender.endRefreshing()
    }
    
    //
    // Mostrar dades habitatge
    //
    func mostrarDadesHabitatge() {
        // Mostrar nom, adreça i població de l'habitatge.
        if ((moHabitatgeActual) != nil) {
            lblNom?.text = moHabitatgeActual?.nom
            lblDireccio?.text = moHabitatgeActual?.direccio
            lblPoblacio?.text = moHabitatgeActual?.poblacio
        }
    }

    //
    // Calcular i mostrar els totals de ingressos, despeses i taxes
    // de l'habitatge actual
    //
    func calcularTotalsHabitatge() {
        
        if (moHabitatgeActual==nil) {
            // Mostrar zeros si no tenim habitatge (pot passar al fer servir iPad en pantalla gran o iphone6S apaisat
            self.lblTotalIngressos?.text = "0.00"
            self.lblTotalDespeses?.text = "0.00"
            self.lblTotalTaxes?.text = "0.00"
            self.lblTotalBenefici?.text = "0.00"
            return
        }
        
        // Carreguem els totals en un thread asincron perquè pot haver-hi moltes dades
        // i afectaria a la usabilitat en primer pla de la interfície.
        let qos = Int(QOS_CLASS_USER_INITIATED.value)
        dispatch_async(dispatch_get_global_queue(qos, 0)) { () -> Void in
           
            // Mostrar que estem fent els càlculs
            self.lblTotalIngressos?.text = NSLocalizedString("Computing ...", comment: "Calculant totals")
            self.lblTotalDespeses?.text = NSLocalizedString("Computing ...", comment: "Calculant totals")
            self.lblTotalTaxes?.text = NSLocalizedString("Computing ...", comment: "Calculant totals")
            self.lblTotalBenefici?.text = NSLocalizedString("Computing ...", comment: "Calculant totals")
            
            // Procés lent (càlcul i recorregut de dades)
            // Ingressos
            self.totalIngressos = self.calcularTotalIngressos()
            // Despeses
            self.totalDespeses = self.calcularTotalDespeses()
            // Taxes
            self.totalTaxes = self.calcularTotalTaxes()

            // La part que afecta a la resposta i es mostra al UI de l'usari , cal tornar-la a
            // dispatch en el "main queue".
            dispatch_async(dispatch_get_main_queue()) {
        
                // Cal tenir en compte la localització (per exemple en català/español el decimal és una coma, no un punt. Afagem el currentLocale de l'aparell de l'usuari
                var formatter = NSNumberFormatter()
                formatter.locale = NSLocale.currentLocale()
                formatter.numberStyle = NSNumberFormatterStyle.CurrencyStyle
                // Màxim dos decimals als imports
                formatter.maximumFractionDigits = 2
                
                // Ingressos
                self.lblTotalIngressos?.text = formatter.stringFromNumber(self.totalIngressos)
                // Despeses (canviant el signe)
                self.lblTotalDespeses?.text = formatter.stringFromNumber(self.totalDespeses.decimalNumberByMultiplyingBy(-1))
                // Taxes (canviant el signe)
                self.lblTotalTaxes?.text = formatter.stringFromNumber(self.totalTaxes.decimalNumberByMultiplyingBy(-1))
 
                // Total benefici
                self.totalBenefici = self.totalIngressos
                self.totalBenefici = self.totalBenefici.decimalNumberBySubtracting(self.totalDespeses)
                self.totalBenefici = self.totalBenefici.decimalNumberBySubtracting(self.totalTaxes)
                self.lblTotalBenefici?.text = formatter.stringFromNumber(self.totalBenefici)
                
            }
        }

    }
    
    //
    // Calcular els ingressos
    //
    func calcularTotalIngressos() -> NSDecimalNumber {
        
        // Calcular ingressos recorrent les entitats dins de l'objecte moHabitatgeActual
        
        // Array de objectes NSManagedObject de la entity Ingres (Classe Ingressos)
        var moIngressos = [Ingressos]()
        // Variable on acumularem el total
        var totalIngressos: NSDecimalNumber = NSDecimalNumber.zero()
        
        // Carregar els ingressos que tenim a l'objecte de l'habitatge actual
        // Cal convertir el tipus NSSet dels ingressos (relació un-a-molts del core data)
        // a un tipus Array. "allOjbects" ho fa directament.
        moIngressos = moHabitatgeActual?.ingressos.allObjects as [Ingressos]
        for ingres:Ingressos in moIngressos {
            // Acumular import total
            totalIngressos = totalIngressos.decimalNumberByAdding(ingres.importcobrat)
        }
        return totalIngressos
    }
    
    //
    // Calcular les despeses
    //
    func calcularTotalDespeses() -> NSDecimalNumber {
        
        // Calcular despeses recorrent les entitats dins de l'objecte moHabitatgeActual
        
        // Array de objectes NSManagedObject de la entity Despesa (Classe Despeses)
        var moDespeses = [Despeses]()
        // Variable on acumularem el total
        var totalDespeses: NSDecimalNumber = NSDecimalNumber.zero()
        
        // Carregar les despeses que tenim a l'objecte de l'habitatge actual
        // Cal convertir el tipus NSSet de les despeses (relació un-a-molts del core data)
        // a un tipus Array. "allOjbects" ho fa directament.
        moDespeses = moHabitatgeActual?.despeses.allObjects as [Despeses]
        for despesa:Despeses in moDespeses {
            // Acumular import total
            totalDespeses = totalDespeses.decimalNumberByAdding(despesa.importpagat)
        }
        return totalDespeses
    }

    //
    // Calcular les taxes
    //
    func calcularTotalTaxes() -> NSDecimalNumber {
        
        // Calcular taxes dels ingressos recorrent les entitats Ingres dins de l'objecte moHabitatgeActual
        
        // Array de objectes NSManagedObject de la entity Ingres (Classe Ingressos)
        var moIngressos = [Ingressos]()
        // Variables on acumularem els totals
        var taxaPersonaNit: NSDecimalNumber
        var taxaDataValidesa: NSDate?
        var nits: NSDecimalNumber
        var adults: NSDecimalNumber
        var totalTaxes: NSDecimalNumber = NSDecimalNumber.zero()
        
        // Carregar les tarifes i dates de validesa de les taxes de la zona de l'habitatge
        var requestConfiguracio = NSFetchRequest(entityName: "Configuracio")
        requestConfiguracio.returnsObjectsAsFaults = false
        var moTaxesConfig: [Configuracio] = moContext.executeFetchRequest(requestConfiguracio, error: nil)! as [Configuracio]
        
        // Carregar els ingressos que tenim a l'objecte de l'habitatge actual
        // Cal convertir el tipus NSSet dels ingressos (relació un-a-molts del core data)
        // a un tipus Array. "allOjbects" ho fa directament.
        moIngressos = moHabitatgeActual?.ingressos.allObjects as [Ingressos]
        for ingres:Ingressos in moIngressos {

            // Buscar quin import de taxa li correspon per la zona i la data
            // La taxa que correspon a l'ingrés és la que té data de validesa inmediatament inferior 
            // a la data d'ingrés
            taxaDataValidesa = nil
            taxaPersonaNit = NSDecimalNumber.zero()
            for taxaConfig in moTaxesConfig {

                if (taxaConfig.zona == moHabitatgeActual?.zona) {
                    if (taxaDataValidesa==nil || ( ingres.dataentrada.compare(taxaConfig.datavalidesa)==NSComparisonResult.OrderedDescending && (taxaConfig.datavalidesa).compare(taxaDataValidesa!)==NSComparisonResult.OrderedDescending ) ) {
                        taxaPersonaNit = taxaConfig.taxapersonanit
                        taxaDataValidesa = taxaConfig.datavalidesa
                    }
                }
            }

            // Acumular import total de les taxes
            nits = NSDecimalNumber(double: ingres.nits as Double)
            adults = NSDecimalNumber(double: ingres.numadults as Double)
            totalTaxes = totalTaxes.decimalNumberByAdding( (adults.decimalNumberByMultiplyingBy(nits)).decimalNumberByMultiplyingBy(taxaPersonaNit) )

        }
        return totalTaxes
    }
    

    // MARK: - Table view data source

    // Retornar el total de seccions
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 2
    }

    // Retornar el total de files per la secció indicada
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        switch section {
            case 0:
                // 1 fila per mostrar info habitatge
                return 1
            case 1:
                // 3 files per mostrar menu (ingressos, depeses i taxes)
                return 3
            default:
                return 0
        }
    }

    // Assignar estil (color) al fons de les seccions
    override func tableView(tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        var header :UITableViewHeaderFooterView = UITableViewHeaderFooterView()
        header.contentView.backgroundColor = UIColor(red: 108.0/255.0, green: 226.0/255.0, blue: 108.0/255.0, alpha: 0.5)
        return header
    }

    // MARK: - Navigation
    

    // Validacions abans de mostrar llistes de ingressos/despeses/taxes
    override func shouldPerformSegueWithIdentifier(identifier: String!, sender: AnyObject!) -> Bool {
        // Si no tenim cap habitatge seleccionat (splitview) no mostrem el segue
        if (self.moHabitatgeActual==nil) {
            return false
        }
        return true
    }
    

    //
    // Atendre les peticions dels Segue que tenim definits a l'storyboard
    //
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        var destination = segue.destinationViewController as? UIViewController

        // La vista per editar un habitatge està embebida dins un UINavigationController
        // Per tant cal allargar el segue i saltar el UINav
        if let navcon = destination as? UINavigationController {
            destination = navcon.visibleViewController
        }
        
        // Si el segue apunta a la fitxa per editar habitatge
        if let vc = destination as? HabitatgeEditarTableViewController {
            if let identifier = segue.identifier {
                switch identifier {
                case "EditarHabitatge":
                    // Carregar objecte de l'habitatge seleccionat al MVC que edita les dades
                    vc.moHabitatgeActual = self.moHabitatgeActual
                    break
                default:
                    break
                }
            }
        }

        // Si el segue apunta a la llista d'ingressos de l'habitatge seleccionat
        if let vc = destination as? IngressosLlistaTableViewController {
            if let identifier = segue.identifier {
                switch identifier {
                case "LlistaIngressos":
                    // Carregar objecte de l'habitatge seleccionat al MVC que mostra els ingressos
                    vc.moHabitatgeActual = self.moHabitatgeActual
                    break
                default:
                    break
                }
            }
        }
        
        // Si el segue apunta a la llista de despeses de l'habitatge seleccionat
        if let vc = destination as? DespesesLlistaTableViewController {
            if let identifier = segue.identifier {
                switch identifier {
                case "LlistaDespeses":
                    // Carregar objecte de l'habitatge seleccionat al MVC que mostra les despeses
                    vc.moHabitatgeActual = self.moHabitatgeActual
                    break
                default:
                    break
                }
            }
        }
        
        // Si el segue apunta a la llista de taxes trimestral de l'habitatge seleccionat
        if let vc = destination as? TaxesLlistaTableViewController {
            if let identifier = segue.identifier {
                switch identifier {
                case "LlistaTaxes":
                    // Carregar objecte de l'habitatge seleccionat al MVC que mostra les taxes per trimestres
                    vc.moHabitatgeActual = self.moHabitatgeActual
                    break
                default:
                    break
                }
            }
        }

    }

    // Funció per atendre els unwind segue (els segue de retorn)
    @IBAction func unwindEditarHabitatge(segue: UIStoryboardSegue) {
        var vcHabitatgeEditar = segue.sourceViewController as HabitatgeEditarTableViewController
        var moHabitatgeEditada: Habitatges? = vcHabitatgeEditar.moHabitatgeActual
        
        self.moHabitatgeActual = moHabitatgeEditada
        if let identifier = segue.identifier {
            switch identifier {
            case "HabitatgeCancelar":
                // Si l'usuari ha cancelat l'edició de l'habitatge no cal fer res més
                break
            case "HabitatgeGuardar":
                // Refrescar dades habitatge
                mostrarDadesHabitatge()
                break
            default:
                break
            }
        }
        
        // Tancar la pantalla d'editar (S'ha mostrat en iPad en format formsheet i cal tancar-la)
        vcHabitatgeEditar.dismissViewControllerAnimated(true, completion: nil)
    }

}
