//
//  HabitatgesMasterTableViewController.swift
//  LloguersTuristics
//
//  Created by Manel Juarez on 8/3/15.
//  Copyright (c) 2015 Manel Juarez. All rights reserved.
//

//
// Agraiments a aquests 2 bloguers pels exemples i la informació sobre NSFetchedResultsController
// Font1: http://www.pumpmybicep.com/2014/07/07/nsfetchedresultscontroller-ios-core-data-basics-in-swift/
// Font2: http://www.andrewcbancroft.com/2015/03/05/displaying-data-with-nsfetchedresultscontroller-and-swift/
//

import UIKit
import CoreData

class HabitatgesMasterTableViewController: UITableViewController, UISplitViewControllerDelegate, NSFetchedResultsControllerDelegate {

    // Obtenir el Context per treballar amb el Core Data
    let moContext = (UIApplication.sharedApplication().delegate as AppDelegate).managedObjectContext!
    
    // El controlador que obté la informació (NSFetchedResultsController)
    // S'inicialitza de forma passiva "lazy", és dir es carregarà tot el primer cop que es faci referencia a la variable. Carregarà tots els habitatges de la base de dades
    // L'ordenació és obligatoria i ho fem per Nom.
    var fetchedResultsController: NSFetchedResultsController {
        
        // Retornem ell mateix si ja està creat (Patró Singleton)
        if self._fetchedResultsController != nil {
            return self._fetchedResultsController!
        }
        
        // Carregarem les dades d'habitatges, ordenades per nom ascendent
        let entity = NSEntityDescription.entityForName("Habitatges", inManagedObjectContext: moContext)
        let sort = NSSortDescriptor(key: "nom", ascending: true)
        let request = NSFetchRequest()
        request.entity = entity
        request.sortDescriptors = [sort]
        
        // Aquesta llista no s'agrupa per seccions, per tant el sectionNameKeyPath es posa a nil
        let aFetchedResultsController = NSFetchedResultsController(fetchRequest: request, managedObjectContext: moContext, sectionNameKeyPath: nil, cacheName: nil)
        
        // El delegate per gestionar les actualitzacions serà aquest mateix controller
        aFetchedResultsController.delegate = self
        
        // Guardem els resultats
        self._fetchedResultsController = aFetchedResultsController
        
        // Llancem la primera petició de dades, controlant els errors
        var e: NSError?
        if !self._fetchedResultsController!.performFetch(&e) {
            println(NSLocalizedString("Error loading data:", comment: "Errors control") + " \(e!.localizedDescription)")
            abort();
        }
        
        // Retornem els resultats
        return self._fetchedResultsController!
    }
    // Variable per gestionar el patró singleton
    var _fetchedResultsController: NSFetchedResultsController?

    
   
    // Propietat per determinar si es permet amagar o no la vista detall
    // Té relació en la gestió del splitViewController en iphone/ipad.
    private var collapseDetailViewController = true
    
    
    override func awakeFromNib() {
        super.awakeFromNib()
        if UIDevice.currentDevice().userInterfaceIdiom == .Pad {
            // No borrar la fila seleccionada quan la tableView apareix.
            self.clearsSelectionOnViewWillAppear = false
            self.preferredContentSize = CGSize(width: 320.0, height: 600.0)
        }
    }
 
    // Flag per seleccionar la primera fila només una vegada
    var seleccPrimeraFila: Bool = true
    
    // Funció que s'executa quan s'ha carregat el MVC
    override func viewDidLoad() {
        super.viewDidLoad()

        // Configurar el delegate del splitView en aquest mateix controller
        splitViewController?.delegate = self
        //navigationController?.topViewController.navigationItem.leftBarButtonItem = splitViewController?.displayModeButtonItem()
        
        // Amb aquesta comanda obliguem a que tant el Master com el Detail estiguin sempre visibles
        // inclús en la disposició iPad Portrait (vertical), en què per defecte el Master s'oculta a 
        // la part esquerra per defecte.
        self.splitViewController?.preferredDisplayMode = UISplitViewControllerDisplayMode.AllVisible
        
        // Afegir dades de prova si no hi ha cap registre creat a la base de dades
        crearDadesExempleSiCal()

        // Mantenir la selecció entre diferents presentacions
        self.clearsSelectionOnViewWillAppear = false
        
        // Seleccionar el primer habitatge en ipad
        // Si estem en iPad o en amplada de pantalla >= 736 (iphone 6s apaisat)
        // Ja podem carregar la part del detall perquè estarà visible
        let screenSize: CGRect = UIScreen.mainScreen().bounds
        let screenWidth = screenSize.width
        if ( UIDevice.currentDevice().userInterfaceIdiom == .Pad || screenWidth>=736) {
            if ( seleccPrimeraFila) {
                seleccPrimeraFila = false
                let rowToSelect:NSIndexPath = NSIndexPath(forRow: 0, inSection: 0)
                self.tableView.selectRowAtIndexPath(rowToSelect, animated: true, scrollPosition:UITableViewScrollPosition.None)
                // Mostrar el detall ja que el splitview manté les dues pantalles obertes en ipad
                self.performSegueWithIdentifier("DetallHabitatge", sender: self)
            }
        }
    }

    // Fem servir el control refresh "pull down" i que recarregui la informació de la llista
    @IBAction func refrescarLlista(sender: UIRefreshControl) {
        // Provocar que el tableView torni a demanar les dades
        tableView.reloadData()
        // Amagar la imatge "spinning-carregant"
        sender.endRefreshing()
    }
    
    // MARK: - Table view data source

    // Retornar el total de seccions
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        // Demanem al controller que gestiona les dades quantes seccions hi ha
        if let sections = self.fetchedResultsController.sections {
            return sections.count
        }
        return 0
    }

    // Retornar el total de registres en una secció
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        // Demanem al controller que gestiona les dades quants registres hi ha
        if let sections = self.fetchedResultsController.sections {
            let currentSection = sections[section] as NSFetchedResultsSectionInfo
            return currentSection.numberOfObjects
        }
        return 0
    }

    // Petició de dades per una cel·la concreta
    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        // Demanar una cel·la que estigui lliure o sinó en crearà una de nova del tipus "InfoHabitatge"
        let cell = tableView.dequeueReusableCellWithIdentifier("InfoHabitatge", forIndexPath: indexPath) as UITableViewCell
        
        // Configurar la cel·la
        self.configurarCell(cell, atIndexPath: indexPath)
        return cell
    }

    // Configurar (carregar dades) a una cel·la a partir d'un registre determinat (NSIndexPath) del NSFetchedResultsController
    func configurarCell(cell: UITableViewCell,
        atIndexPath indexPath: NSIndexPath) {
    
        // Carregar i fer el cast del l'habitatge seleccionat
        let item = self.fetchedResultsController.objectAtIndexPath(indexPath) as Habitatges
            
        // Mostrar nom i població a la cel·la
        cell.textLabel?.text = item.nom
        cell.detailTextLabel?.text = item.poblacio
    }
    
    
    //
    // Gestió de les actualitzacions del "fetched results controller delegate"
    //
    
    // Es crida al principi
    // Serveix per assegurar que totes les actualitzacions apliquen l'animació a la vegada
    func controllerWillChangeContent(controller: NSFetchedResultsController) {
        self.tableView.beginUpdates()
    }
    
    // Es crida quan:
    // Es crea un nou model (nou habitatge)
    // S'actualitza un model ja existent (modificar/editar habitatge)
    // S'elimina un model (esborrar habitatge)
    func controller(controller: NSFetchedResultsController,
        didChangeObject object: AnyObject,
        atIndexPath indexPath: NSIndexPath,
        forChangeType type: NSFetchedResultsChangeType,
        newIndexPath: NSIndexPath) {
            switch type {
            case .Insert:
                self.tableView.insertRowsAtIndexPaths([newIndexPath], withRowAnimation: .Fade)
            case .Update:
                let cell = self.tableView.cellForRowAtIndexPath(indexPath)
                self.configurarCell(cell!, atIndexPath: indexPath)
                self.tableView.reloadRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
            // No permetem moure (reordenar habitatges) per tant aquesta part no cal
            //case .Move:
            //    self.tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
            //    self.tableView.insertRowsAtIndexPaths([newIndexPath], withRowAnimation: .Fade)
            case .Delete:
                self.tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
            default:
                return
            }
    }
    
    // Es crida al final
    // Indica al UITableView que s'han completat les actualitzacions
    func controllerDidChangeContent(controller: NSFetchedResultsController) {
        self.tableView.endUpdates()
    }

    
    // Controlar l'eliminació d'un registre fent tap+swipe esquerra (apareix botó "eliminar")
    override func tableView(tableView: UITableView, commitEditingStyle editingStyle: UITableViewCellEditingStyle, forRowAtIndexPath indexPath: NSIndexPath) {
        
        if editingStyle == .Delete {
            // Esborrar habitatge del core data
           if let habitatgeSelec = self.fetchedResultsController.objectAtIndexPath(indexPath) as? Habitatges {
                // Esborrar del context core data
                moContext.deleteObject(habitatgeSelec)
                // Guardar context
                var errorGuardar: NSError? = nil
                if !moContext.save(&errorGuardar) {
                    println(NSLocalizedString("Error deleting accommodation:", comment: "Errors control") + "  \(errorGuardar!.localizedDescription)")
                    abort();
                }
            }
        }
    }

    // MARK: - Navigation
    
    //
    // Atendre les peticions dels Segue que tenim definits a l'storyboard
    //
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        var destination = segue.destinationViewController as? UIViewController
        
        // La vista per editar un habitatge està embebida dins un UINavigationController
        // Per tant cal allargar el segue i saltar el UINav
        if let navcon = destination as? UINavigationController {
            destination = navcon.visibleViewController
        }

        // Si el segue apunta a mostrar el detall de l'habitatge
        if let vc = destination as? HabitatgesDetailTableViewController {
            // Obtenir la fila del tableview que està seleccionada
            if let indexPath = self.tableView.indexPathForSelectedRow() {
                // Obtenir l'objecte habitatge que correspon a la fila seleccionada
                let habitatgeSelec = self.fetchedResultsController.objectAtIndexPath(indexPath) as Habitatges
                if let identifier = segue.identifier {
                    switch identifier {
                        case "DetallHabitatge":
                            // Carregar objecte de l'habitatge seleccionat al MVC que mostra els detalls
                            vc.moHabitatgeActual = habitatgeSelec
                            break
                        default:
                            break
                    }
                }
            }
        }

        // Si el segue apunta a la fitxa per crear nou habitatge
        if let vc = destination as? HabitatgeEditarTableViewController {
            if let identifier = segue.identifier {
                switch identifier {
                    case "NouHabitatge":
                        vc.moHabitatgeActual = nil
                        break
                    default:
                        break
                }
            }
        }
    }

    // Funció per atendre els unwind segue (els segue de retorn)
    @IBAction func unwindEditarHabitatge(segue: UIStoryboardSegue) {
        if let vcHabitatgeEditar = segue.sourceViewController as? HabitatgeEditarTableViewController {
            // Tancar la pantalla d'editar (S'ha mostrat en iPad en format formsheet i cal tancar-la)
            vcHabitatgeEditar.dismissViewControllerAnimated(true, completion: nil)
        }
    }

    // MARK: UITableViewDelegate
    
    override func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        collapseDetailViewController = false
    }
    
    // MARK: - UISplitViewControllerDelegate
    
    func splitViewController(splitViewController: UISplitViewController, collapseSecondaryViewController secondaryViewController: UIViewController!, ontoPrimaryViewController primaryViewController: UIViewController!) -> Bool {
        
        return collapseDetailViewController
    }
    
    

    //
    // Funció per crear dades d'exemple
    // 
    func crearDadesExempleSiCal() {
        
        var continuar: Bool = true
        
        // Primer mirem si ja hi ha dades a la secció 0 (única), per no crear les d'exemple
        if let sections = self.fetchedResultsController.sections {
            let currentSection = sections[0] as NSFetchedResultsSectionInfo
            if (currentSection.numberOfObjects>0) {
                continuar = false
            }
        }
        
        if (continuar) {

            // Crear un objecte per formatar les dates d'entrada i de factura
            let dateformatter: NSDateFormatter = NSDateFormatter()
            dateformatter.dateFormat = "dd-MM-yyyy"

            // Obtenir el entity Habitatges
            let entityHabitatges = NSEntityDescription.entityForName("Habitatges", inManagedObjectContext: moContext)!
        
            // Afegir habitatge d'exemple
            var habitatge = Habitatges(entity: entityHabitatges, insertIntoManagedObjectContext: moContext)

            habitatge.nom = NSLocalizedString("Example 1", comment: "Example data accommodation name")
            habitatge.direccio = "Pl. Sant Jaume 1"
            habitatge.poblacio = "Barcelona"
            habitatge.zona = 1
            
            // Afegir ingressos d'exemple
            let entityIngressos = NSEntityDescription.entityForName("Ingressos", inManagedObjectContext: moContext)!
        
            var ingres = Ingressos(entity: entityIngressos, insertIntoManagedObjectContext: moContext)
            ingres.dataentrada = dateformatter.dateFromString("01-03-2015")!
            ingres.nomllogater = "Jordi Pujol"
            ingres.numadults = 2
            ingres.numnens = 7
            ingres.nits = 23
            ingres.importcobrat = 2500.00
            ingres.habitatge = habitatge
        
            ingres = Ingressos(entity: entityIngressos, insertIntoManagedObjectContext: moContext)
            ingres.dataentrada = dateformatter.dateFromString("20-03-2015")!
            ingres.nomllogater = "Pascual Maragall"
            ingres.numadults = 1
            ingres.numnens = 3
            ingres.nits = 3
            ingres.importcobrat = 500.00
            ingres.habitatge = habitatge
            
            ingres = Ingressos(entity: entityIngressos, insertIntoManagedObjectContext: moContext)
            ingres.dataentrada = dateformatter.dateFromString("15-04-2015")!
            ingres.nomllogater = "José Montilla"
            ingres.numadults = 1
            ingres.numnens = 2
            ingres.nits = 4
            ingres.importcobrat = 750.00
            ingres.habitatge = habitatge

            ingres = Ingressos(entity: entityIngressos, insertIntoManagedObjectContext: moContext)
            ingres.dataentrada = dateformatter.dateFromString("20-03-2015")!
            ingres.nomllogater = "Artur Mas"
            ingres.numadults = 2
            ingres.numnens = 1
            ingres.nits = 5
            ingres.importcobrat = 1000.00
            ingres.habitatge = habitatge
            
            // Afegir despeses d'exemple
            let entityDespeses = NSEntityDescription.entityForName("Despeses", inManagedObjectContext: moContext)!
            
            var despesa = Despeses(entity: entityDespeses, insertIntoManagedObjectContext: moContext)
            despesa.datafactura = dateformatter.dateFromString("31-01-2015")!
            despesa.concepte = NSLocalizedString("Internet ADSL", comment: "Example data expense")
            despesa.importpagat = 30.00
            despesa.habitatge = habitatge
            
            despesa = Despeses(entity: entityDespeses, insertIntoManagedObjectContext: moContext)
            despesa.datafactura = dateformatter.dateFromString("31-01-2015")!
            despesa.concepte = NSLocalizedString("Cleaning", comment: "Example data expense")
            despesa.importpagat = 50.00
            despesa.habitatge = habitatge
            
            despesa = Despeses(entity: entityDespeses, insertIntoManagedObjectContext: moContext)
            despesa.datafactura = dateformatter.dateFromString("28-02-2015")!
            despesa.concepte = NSLocalizedString("Internet ADSL", comment: "Example data expense")
            despesa.importpagat = 30.00
            despesa.habitatge = habitatge
            
            despesa = Despeses(entity: entityDespeses, insertIntoManagedObjectContext: moContext)
            despesa.datafactura = dateformatter.dateFromString("31-03-2015")!
            despesa.concepte = NSLocalizedString("Internet ADSL", comment: "Example data expense")
            despesa.importpagat = 30.00
            despesa.habitatge = habitatge
            
            despesa = Despeses(entity: entityDespeses, insertIntoManagedObjectContext: moContext)
            despesa.datafactura = dateformatter.dateFromString("31-01-2015")!
            despesa.concepte = NSLocalizedString("Electricity", comment: "Example data expense")
            despesa.importpagat = 15.52
            despesa.habitatge = habitatge
            
            // Afegir zones taxa turística d'exemple (Barcelona i resta catalunya)
            let entityConfiguracio = NSEntityDescription.entityForName("Configuracio", inManagedObjectContext: moContext)!
            
            var conf = Configuracio(entity: entityConfiguracio, insertIntoManagedObjectContext: moContext)
            conf.datavalidesa = dateformatter.dateFromString("01-01-2012")!
            conf.zona = 1
            conf.taxapersonanit = 0.65

            conf = Configuracio(entity: entityConfiguracio, insertIntoManagedObjectContext: moContext)
            conf.datavalidesa = dateformatter.dateFromString("01-01-2012")!
            conf.zona = 2
            conf.taxapersonanit = 0.45

            // Guardar context
            var errorGuardar: NSError? = nil
            if !moContext.save(&errorGuardar) {
                println(NSLocalizedString("Error saving accommodations example data:", comment: "Errors control") + "  \(errorGuardar!.localizedDescription)")
                abort();
            }
        }
            
    }
    
}
