//
//  IngresEditarTableViewController.swift
//  LloguersTuristics
//
//  Created by Manel Juarez on 4/4/15.
//  Copyright (c) 2015 Manel Juarez. All rights reserved.
//

import UIKit
import CoreData

class IngresEditarTableViewController: UITableViewController, UITextFieldDelegate {

    // Obtenir el Context per el Core Data
    let moContext = (UIApplication.sharedApplication().delegate as AppDelegate).managedObjectContext!
    
    // Objecte que conté les dades del habitatge seleccionat.
    var moHabitatgeActual: Habitatges?
    
    // Objecte que conté les dades de l'ingrés actual que estem editant
    // Si el valor és nil significa que estem creant un nou ingrés
    // És el Model d'aquest MVC
    var moIngresActual: Ingressos?
    
    // Botons per guardar i cancelar
    @IBOutlet weak var btnGuardar: UIBarButtonItem!
    @IBOutlet weak var btnCancelar: UIBarButtonItem!
    
    // Textfields per editar valors que tenim definits a la pantalla d'edició
    @IBOutlet weak var tfData: UITextField!
    @IBOutlet weak var tfNomllogater: UITextField!
    @IBOutlet weak var tfNumadults: UITextField!
    @IBOutlet weak var tfNumnens: UITextField!
    @IBOutlet weak var tfNits: UITextField!
    @IBOutlet weak var tfImportcobrat: UITextField!
    
    
    // Steppers que tenim definits per els camps numèrics
    @IBOutlet weak var stepperAdults: UIStepper!
    @IBOutlet weak var stepperNens: UIStepper!
    @IBOutlet weak var stepperNits: UIStepper!
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Evitar que el tableview mostri les files en blanc al final, ja que és un model estàtic
        // per simular un formulari d'edició
        tableView.tableFooterView = UIView(frame: CGRect.zeroRect)
        
        // Assignar el delegate dels textfield, per poder gestionar el return = next etc...
        tfData.delegate = self
        tfNomllogater.delegate = self
        tfNumadults.delegate = self
        tfNumnens.delegate = self
        tfNits.delegate = self
        tfImportcobrat.delegate = self
        
        // Carregar els camps amb els valors de l'ingrés si estem modificant.
        // La data de l'ingrés s'ha de formatar per poder-la mostrar
        var dateformatter: NSDateFormatter = NSDateFormatter()
        dateformatter.dateStyle = NSDateFormatterStyle.LongStyle
        dateformatter.locale = NSLocale.currentLocale()
        
        // Cal tenir en compte la localització (per exemple en català/español el decimal és una coma, no un punt. Afagem el currentLocale de l'aparell de l'usuari
        var formatter = NSNumberFormatter()
        formatter.locale = NSLocale.currentLocale()
        formatter.numberStyle = NSNumberFormatterStyle.DecimalStyle
        // Màxim dos decimals als imports
        formatter.maximumFractionDigits = 2
        
        if ((moIngresActual) != nil) {
            var dataentrada = moIngresActual?.dataentrada
            tfData?.text = dateformatter.stringFromDate(dataentrada!) as String?
            tfNomllogater?.text = moIngresActual?.nomllogater
            tfNumadults?.text = moIngresActual?.numadults.stringValue
            tfNumnens?.text = moIngresActual?.numnens.stringValue
            tfNits?.text = moIngresActual?.nits.stringValue
            tfImportcobrat?.text = formatter.stringFromNumber(moIngresActual?.importcobrat as Double)
            
            // Carregar el valor dels steppers
            stepperAdults.value = moIngresActual?.numadults as Double
            stepperNens.value = moIngresActual?.numnens as Double
            stepperNits.value = moIngresActual?.nits as Double
            
        } else {
            
            // Si estem en mode crear
            // Posem data rellotge i altres valors per defecte
            tfData?.text = dateformatter.stringFromDate(NSDate()) as String?
            tfNomllogater?.text = ""
            tfNumadults?.text = "0"
            tfNumnens?.text = "0"
            tfNits?.text = "1"
            stepperNits.value = 1
            
            // passem focus al segon camp perquè desplegui el teclat
            tfNomllogater.becomeFirstResponder()
        }
    }

    // MARK: - Table view data source

    // Retornar el total de seccions
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }

    // Retornar el total de files per la secció indicada
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return 1
    }

    // Assignar títol i estil (color) al fons de la única secció
    override func tableView(tableView: UITableView, willDisplayHeaderView view: UIView, forSection section: Int) {
        
        // Fem un cast per el tipus de la capçalera
        let header: UITableViewHeaderFooterView = view as UITableViewHeaderFooterView
        
        if (section==0) {
            // Posar colo de fons
            header.contentView.backgroundColor = UIColor(red: 108.0/255.0, green: 226.0/255.0, blue: 108.0/255.0, alpha: 0.5)
            // Posar nom de l'habitatge com a títol
            header.textLabel.text = moHabitatgeActual?.nom
        }
    }
    
    
    // El camp de la data d'entrada l'editarem amb un Date Picker
    @IBAction func tfDataEditDidBegin(sender: UITextField) {
        var datePickerView:UIDatePicker = UIDatePicker()
        datePickerView.datePickerMode = UIDatePickerMode.Date
        
        // Assignar la data actual del camp en pantalla al picker
        var dateformatter: NSDateFormatter = NSDateFormatter()
        dateformatter.dateStyle = NSDateFormatterStyle.LongStyle
        dateformatter.locale = NSLocale.currentLocale()
        datePickerView.date = dateformatter.dateFromString(tfData.text)!
        
        sender.inputView = datePickerView
        datePickerView.addTarget(self, action: Selector("datePickerValueChanged:"), forControlEvents: UIControlEvents.ValueChanged)
    }
    
    // Un cop seleccionada una data amb el Date Picker l'assignem al UITextfield associat
    func datePickerValueChanged(sender: UIDatePicker) {
        var dateformatter = NSDateFormatter()
        dateformatter.dateStyle = NSDateFormatterStyle.LongStyle
        dateformatter.locale = NSLocale.currentLocale()
        tfData.text = dateformatter.stringFromDate(sender.date)
    }
    
    // Gestionar els steppers per modificar els camps numèrics
    @IBAction func stepperAdultsChange(sender: UIStepper) {
        self.tfNumadults.text = Int(sender.value).description
        amagarTeclat()
    }
    
    @IBAction func stepperNensChange(sender: UIStepper) {
        self.tfNumnens.text = Int(sender.value).description
        amagarTeclat()
    }
    
    @IBAction func stepperNitsChange(sender: UIStepper) {
        self.tfNits.text = Int(sender.value).description
        amagarTeclat()
    }
    
    //
    // Amagar teclat
    //
    func amagarTeclat() {
        tfData.resignFirstResponder()
        tfNomllogater.resignFirstResponder()
        tfImportcobrat.resignFirstResponder()
    }
    
    //
    // Gestionar el botó return del teclat per canviar de camp
    //
    func textFieldShouldReturn(textField: UITextField) -> Bool {
        switch textField {
        case self.tfData:
            self.tfNomllogater.becomeFirstResponder()
            break
        case self.tfNomllogater:
            self.tfImportcobrat.becomeFirstResponder()
            break
        default:
            break
        }
        return true
    }

    // Els camps que tenen un stepper associat no els editem manualment
    func textFieldShouldBeginEditing(textField: UITextField) -> Bool {
        switch textField {
        case self.tfNumadults:
            return false
        case self.tfNumnens:
            return false
        case self.tfNits:
            return false
        default:
            return true
        }
    }

    // MARK: - Navigation

    // Validacions abans de guardar
    override func shouldPerformSegueWithIdentifier(identifier: String!, sender: AnyObject!) -> Bool {
        
        if (sender as UIBarButtonItem == self.btnGuardar) {
            
            // La data no cal validar-la, s'introdueix amb picker i té valor per defecte.
            // Els totals adults, nens i nits no cal validar-los el input es gestiona amb "steppers"
            
            // Validar camp text nom llogater (no pot estar buit)
            if (tfNomllogater.text.isEmpty || tfNomllogater.text==nil) {
                // Concepte incorrecte
                let alert = UIAlertView()
                alert.title = NSLocalizedString("Validation", comment: "Validació nom llogater incorrecte")
                alert.message = NSLocalizedString("Renter name cannot be empty", comment: "Validació nom llogater incorrecte")
                alert.addButtonWithTitle("Ok")
                alert.show()
                // Validació incorrecte
                return false
            }
            
            // Validar format del import (separador decimal, només números , etc...)
            // Cal tenir en compte la localització per el decimal
            var formatter = NSNumberFormatter()
            formatter.locale = NSLocale.currentLocale()
            formatter.numberStyle = NSNumberFormatterStyle.DecimalStyle
            // Màxim 2 decimals
            formatter.maximumFractionDigits = 2
            // Convertir de cadena a tipus NSDecimalNumber per guardar al Core Data
            if ((formatter.numberFromString(tfImportcobrat.text)) == nil) {
                // Import incorrecte
                let alert = UIAlertView()
                alert.title = NSLocalizedString("Validation", comment: "Validació import")
                alert.message = NSLocalizedString("Incorrect amount", comment: "Validació import incorrecte")
                alert.addButtonWithTitle("Ok")
                alert.show()
                // Validació incorrecte
                return false
            }
        }
        
        // Validació correcte
        return true
    }

    
    //
    // Atendre les peticions dels Segue que tenim definits a l'storyboard
    //
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        
        // Preparar per el segue (unwind) que torna enrera (cancelar o guardar)
        if (sender as UIBarButtonItem == self.btnGuardar) {
            if ((moIngresActual) == nil) {
                // Crear nou model si estem "insertant"
                // Obtenir el entity Ingressos
                let entityIngressos = NSEntityDescription.entityForName("Ingressos", inManagedObjectContext: self.moContext)!
                //  Crear un nou objecte Ingressos
                self.moIngresActual = Ingressos(entity: entityIngressos, insertIntoManagedObjectContext: self.moContext)
                // Vincular amb l'habitatge actual
                self.moIngresActual?.habitatge = self.moHabitatgeActual!
            }
            // Passar les dades dels camps d'edició al model moIngresActual
            // La data de l'ingrés s'ha de formatar per poder-la guardar
            var dateformatter: NSDateFormatter = NSDateFormatter()
            dateformatter.dateStyle = NSDateFormatterStyle.LongStyle
            dateformatter.locale = NSLocale.currentLocale()
            self.moIngresActual?.dataentrada = dateformatter.dateFromString(tfData.text)!
            
            self.moIngresActual?.nomllogater = tfNomllogater.text
            self.moIngresActual?.numadults = tfNumadults.text.toInt()!
            self.moIngresActual?.numnens = tfNumnens.text.toInt()!
            self.moIngresActual?.nits = tfNits.text.toInt()!

            // L'import es valida prèviament per tant el valor entrar és vàlid en aquest punt
            if (tfImportcobrat.text.isEmpty || tfImportcobrat.text==nil) {
                self.moIngresActual?.importcobrat =  0.00
            } else {
                // Cal tenir en compte la localització per el decimal
                var formatter = NSNumberFormatter()
                formatter.locale = NSLocale.currentLocale()
                formatter.numberStyle = NSNumberFormatterStyle.DecimalStyle
                // Màxim 2 decimals
                formatter.maximumFractionDigits = 2
                // Convertir de cadena a tipus NSDecimalNumber per guardar al Core Data
                self.moIngresActual?.importcobrat = NSDecimalNumber(double: formatter.numberFromString(tfImportcobrat.text)!.doubleValue)
            }
            
            
            // Guardar context
            var errorGuardar: NSError? = nil
            if !moContext.save(&errorGuardar) {
                println(NSLocalizedString("Error saving income:", comment: "Errors control") + " \(errorGuardar!.localizedDescription)")
                abort();
            }
            
            // Refrescar l'objecte de l'habitatge actual contra la base de dades
            moContext.refreshObject(moHabitatgeActual!, mergeChanges: true)
        }
    }
    

}
