//
//  TaxesLlistaTableViewController.swift
//  LloguersTuristics
//
//  Created by Manel Juarez on 4/5/15.
//  Copyright (c) 2015 Manel Juarez. All rights reserved.
//

import UIKit
import CoreData

class TaxesLlistaTableViewController: UITableViewController {

    // Obtenir el Context per treballar amb el Core Data
    let moContext = (UIApplication.sharedApplication().delegate as AppDelegate).managedObjectContext!
    
    // Objecte que conté les dades del habitatge actual que estem mostrant.
    var moHabitatgeActual: Habitatges?
    
    // Struct on anirem guardant els valors calculats per cada trimestre
    struct taxaTrimestral {
        var trimestre: String = ""
        var totalNits: NSDecimalNumber = NSDecimalNumber.zero()
        var totalAdultsNits: NSDecimalNumber = NSDecimalNumber.zero()
        var totalNensNits: NSDecimalNumber = NSDecimalNumber.zero()
        var preuNit: NSDecimalNumber = NSDecimalNumber.zero()
        var importTaxa: NSDecimalNumber = NSDecimalNumber.zero()
    }
    
    // Array de objectes taxaTrimestral
    // És el Model d'aquest MVC
    var taxesTrimestrals = [taxaTrimestral]()
    

    override func viewDidLoad() {
        super.viewDidLoad()

        // Afegir botons de navegació per poder tornar enrera
        navigationItem.leftBarButtonItem = splitViewController?.displayModeButtonItem()
        navigationItem.leftItemsSupplementBackButton = true
        
        // Mostrar dades habitatge
        carregarDadesTaxesHabitatge()
    }

    //
    // Fem servir el control refresh per simular un "pull down" i que recarregui la informació de la llista
    //
    func carregarDadesTaxesHabitatge() {
        // mostrar imatge "carregant"
        refreshControl?.beginRefreshing()
        refrescarLlista(refreshControl!)
    }
    
    
    @IBAction func refrescarLlista(sender: UIRefreshControl) {
        // Calcular els trimestres
        calcularTrimestres()
        
        // Provocar que el tableView torni a demanar les dades
        tableView.reloadData()
        
        // Amagar la imatge "spinning-carregant"
        sender.endRefreshing()
    }
    
    //
    // Calcular els resums trimestrals a partir dels ingressos, la zona de l'habitatge i 
    // les configuracions de taxes per data de validesa
    //
    func calcularTrimestres() {
        // Netejar valors actuals
        taxesTrimestrals.removeAll(keepCapacity: false)
        
        // Calcular taxes dels ingressos recorrent les entitats Ingres dins de l'objecte moHabitatgeActual
        
        // Array de objectes NSManagedObject de la entity Ingres (Classe Ingressos)
        var moIngressos = [Ingressos]()
        // Variables on acumularem els totals
        var taxaPersonaNit: NSDecimalNumber
        var taxaDataValidesa: NSDate?
        var trimestre: String
        var nits: NSDecimalNumber
        var adults: NSDecimalNumber
        var nens: NSDecimalNumber
        var importTaxa: NSDecimalNumber = NSDecimalNumber.zero()
        // Variables per gestionar dates
        var calendar = NSCalendar.currentCalendar()
        calendar.locale = NSLocale.currentLocale()
        
        // Carregar les tarifes i dates de validesa de les taxes de la zona de l'habitatge
        var requestConfiguracio = NSFetchRequest(entityName: "Configuracio")
        requestConfiguracio.returnsObjectsAsFaults = false
        var moTaxesConfig: [Configuracio] = moContext.executeFetchRequest(requestConfiguracio, error: nil)! as [Configuracio]
        
        // Carregar els ingressos que tenim a l'objecte de l'habitatge actual
        // Cal convertir el tipus NSSet dels ingressos (relació un-a-molts del core data)
        // a un tipus Array. "allOjbects" ho fa directament.
        moIngressos = moHabitatgeActual?.ingressos.allObjects as [Ingressos]
        for ingres:Ingressos in moIngressos {
            
            // Buscar quin import de taxa li correspon per la zona i la data
            // La taxa que correspon a l'ingrés és la que té data de validesa inmediatament inferior
            // a la data d'ingrés
            taxaDataValidesa = nil
            taxaPersonaNit = NSDecimalNumber.zero()
            for taxaConfig in moTaxesConfig {
                
                if (taxaConfig.zona == moHabitatgeActual?.zona) {
                    if (taxaDataValidesa==nil || ( ingres.dataentrada.compare(taxaConfig.datavalidesa)==NSComparisonResult.OrderedDescending && (taxaConfig.datavalidesa).compare(taxaDataValidesa!)==NSComparisonResult.OrderedDescending ) ) {
                        taxaPersonaNit = taxaConfig.taxapersonanit
                        taxaDataValidesa = taxaConfig.datavalidesa
                    }
                }
            }
            
            // La data de l'ingrés s'ha de formatar per poder-la mostrar com un trimestre
            var dateFormatter: NSDateFormatter = NSDateFormatter()
            dateFormatter.locale = NSLocale.currentLocale()
            // Format per mostrar any i trimestre, p.ex. 2015 1er Trimestre
            dateFormatter.dateFormat = "yyyy / QQQQ"
            trimestre = dateFormatter.stringFromDate(ingres.dataentrada)

            // Calcular totals
            nits = NSDecimalNumber(double: ingres.nits as Double)
            adults = NSDecimalNumber(double: ingres.numadults as Double)
            nens = NSDecimalNumber(double: ingres.numnens as Double)
            importTaxa = (adults.decimalNumberByMultiplyingBy(nits)).decimalNumberByMultiplyingBy(taxaPersonaNit)
            
            // Afegir a l'array de trimestres o sumar si ja tenim el trimestre creat
            // Es té en compte la possibilitat que durant un trimestre pugui haver-hi un canvi
            // del preu de tarifa per nit i es mostra en 2 registres separats per poder
            // complimentar correctament el model 950 de la Generalitat Catalunya
            var trobat: Bool = false
            for idxTrim in indices(taxesTrimestrals) {
                if (taxesTrimestrals[idxTrim].trimestre == trimestre && taxesTrimestrals[idxTrim].preuNit == taxaPersonaNit) {
                    // Acumular totals
                    taxesTrimestrals[idxTrim].totalNits = taxesTrimestrals[idxTrim].totalNits.decimalNumberByAdding(nits)
                    taxesTrimestrals[idxTrim].totalAdultsNits = taxesTrimestrals[idxTrim].totalAdultsNits.decimalNumberByAdding(adults.decimalNumberByMultiplyingBy(nits))
                    taxesTrimestrals[idxTrim].totalNensNits = taxesTrimestrals[idxTrim].totalNensNits.decimalNumberByAdding(nens.decimalNumberByMultiplyingBy(nits))
                    taxesTrimestrals[idxTrim].importTaxa = taxesTrimestrals[idxTrim].importTaxa.decimalNumberByAdding(importTaxa)
                    trobat = true
                    break
                }
            }
            
            // Insertar nou objecte a l'array
            if (!trobat) {
                var nouTrim = taxaTrimestral()
                nouTrim.trimestre = trimestre
                nouTrim.totalNits = nits
                nouTrim.totalAdultsNits = adults.decimalNumberByMultiplyingBy(nits)
                nouTrim.totalNensNits = nens.decimalNumberByMultiplyingBy(nits)
                nouTrim.preuNit = taxaPersonaNit
                nouTrim.importTaxa = importTaxa
                taxesTrimestrals.append(nouTrim)
            }
            
            // Ordenar array de trimestres de forma descendent
            taxesTrimestrals.sort({ $0.trimestre > $1.trimestre })
        }       
    }
    
    // MARK: - Table view data source
    
    // Retornar el total de seccions
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }
    
    // Retornar el total de files per la secció indicada
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return taxesTrimestrals.count
    }
    
    // Configurar la informació que es mostra en una cel·la (estàtica - custom)
    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("TaxaCell", forIndexPath: indexPath) as UITableViewCell
        
        // Configurar la cel·la per mostrar informació del trimestre
        // Com que són cel·les tipus "custom" amb varis labels, cadascun té un número "tag" diferent.
        var lblTrimestre : UILabel? = self.view.viewWithTag(10) as? UILabel
        var lblNits: UILabel? = self.view.viewWithTag(110) as? UILabel
        var lblTotalAdultsNits : UILabel? = self.view.viewWithTag(50) as? UILabel
        var lblZona: UILabel? = self.view.viewWithTag(80) as? UILabel
        var lblPreuNit: UILabel? = self.view.viewWithTag(90) as? UILabel
        var lblTotalNensNits : UILabel? = self.view.viewWithTag(60) as? UILabel
        var lblImportTrimestre : UILabel? = self.view.viewWithTag(70) as? UILabel
        
        lblTrimestre?.text = taxesTrimestrals[indexPath.row].trimestre
        lblNits?.text = taxesTrimestrals[indexPath.row].totalNits.stringValue
        lblTotalAdultsNits?.text = taxesTrimestrals[indexPath.row].totalAdultsNits.stringValue
        lblZona?.text = NSLocalizedString("Tax fee zone", comment: "Taxa zona") + " " + (moHabitatgeActual?.zona)!.stringValue + ":"
        lblTotalNensNits?.text = taxesTrimestrals[indexPath.row].totalNensNits.stringValue
        
        // Cal tenir en compte la localització (per exemple en català/español el decimal és una coma, no un punt. Afagem el currentLocale de l'aparell de l'usuari
        var formatter = NSNumberFormatter()
        formatter.locale = NSLocale.currentLocale()
        formatter.numberStyle = NSNumberFormatterStyle.CurrencyStyle
        
        lblPreuNit?.text = formatter.stringFromNumber(taxesTrimestrals[indexPath.row].preuNit)
        lblImportTrimestre?.text = formatter.stringFromNumber(taxesTrimestrals[indexPath.row].importTaxa)
        
        return cell
    }
    
    // Assignar títol i estil (color) al fons de la única secció
    override func tableView(tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        var header :UITableViewHeaderFooterView = UITableViewHeaderFooterView()
        
        header.textLabel.text = moHabitatgeActual?.nom
        header.contentView.backgroundColor = UIColor(red: 108.0/255.0, green: 226.0/255.0, blue: 108.0/255.0, alpha: 0.5)
        return header
    }
    
}
